import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;


public class ParametriASSGA {
	
	/** Vjerojatnost križanja  na početku izvođenja algoritma */
	private double pocetnaVjerojatnostKrizanja;
	/** Korak s kojim se mijenja vjerojatnost križanja*/
	private double korakVjerojatnostiKrizanja;
	/** Vjerojatnost mutacije na početku izvođenja algoritma */
	private double pocetnaVjerojatnostMutacije;
	/** Korak s kojim se mijenja vjerojatnost mutacije */
	private double korakVjerojatnostiMutacije;
	/** Trenutna vrijednost vjerojatnosti križanja */
	private double vjerojatnostKrizanja;
	/** Trenutna vrijednost vjerojatnosti mutacije */
	private double vjerojatnostMutacije;
	/** Broj jedinki u populaciji */
	private int velicinaPopulacije;
	/** Broj reprodukcija koje se obave algoritmom */
	private int brojReprodukcija;
	
	/**
	 * Konstruktor objekta
	 * 
	 * @param ulaznaDatoteka datoteka iz koje se učitavaju parametri
	 * @throws IOException greška prilikom čitanja iz datoteke
	 */
	public ParametriASSGA(File ulaznaDatoteka) throws IOException{
		super();
		BufferedReader citac = new BufferedReader(new FileReader(ulaznaDatoteka));
		String red = citac.readLine();
		velicinaPopulacije = Integer.parseInt(red);
		red = citac.readLine();
		brojReprodukcija = Integer.parseInt(red);
		red = citac.readLine();
		pocetnaVjerojatnostKrizanja = Double.parseDouble(red);
		red = citac.readLine();
		korakVjerojatnostiKrizanja = Double.parseDouble(red);
		red = citac.readLine();
		pocetnaVjerojatnostMutacije = Double.parseDouble(red);
		red = citac.readLine();
		korakVjerojatnostiMutacije = Double.parseDouble(red);
		vjerojatnostKrizanja = pocetnaVjerojatnostKrizanja;
		vjerojatnostMutacije = pocetnaVjerojatnostMutacije;
	}

	/**
	 * Vraća broj jedinki u populaciji
	 * 
	 * @return broj jedinki
	 */
	public int getVelicinaPopulacije() {
		return velicinaPopulacije;
	}

	/**
	 * Postavlja broj jedinki u populaciji
	 * 
	 * @param velicinaPopulacije broj jedinki populacije
	 */
	public void setVelicinaPopulacije(int velicinaPopulacije) {
		this.velicinaPopulacije = velicinaPopulacije;
	}

	/**
	 * Vraća trenutnu vrijednost vjerojatnosti križanja
	 * 
	 * @return vjerojatnost križanja
	 */
	public double getVjerojatnostKrizanja() {
		return vjerojatnostKrizanja;
	}

	/**
	 * Povećava vjerojatnost križanja za korak križanja
	 */
	public void povecajVjerojatnostKrizanja() {
		this.vjerojatnostKrizanja += this.korakVjerojatnostiKrizanja;
	}
	
	/**
	 * Smanjuje vjerojatnost križanja za korak križanja
	 */
	public void smanjiVjerojatnostKrizanja() {
		this.vjerojatnostKrizanja -= this.korakVjerojatnostiKrizanja;
	}

	/**
	 * Vraća trenutnu vrijednost vjerojatnosti mutacije
	 * 
	 * @return vjerojatnost mutacije
	 */
	public double getVjerojatnostMutacije() {
		return vjerojatnostMutacije;
	}

	/**
	 * Povećava vjerojatnost mutacije za korak mutacije
	 */
	public void povecajVjerojatnostMutacije() {
		this.vjerojatnostMutacije += this.korakVjerojatnostiMutacije;
	}
	
	/**
	 * Smanjuje vjerojatnost mutacije za korak mutacije
	 */
	public void smanjiVjerojatnostMutacije() {
		this.vjerojatnostMutacije -= this.korakVjerojatnostiMutacije;
	}

	/**
	 * Vraća broj reprodukcija algoritma
	 * 
	 * @return broj reprodukcija
	 */
	public int getBrojReprodukcija() {
		return brojReprodukcija;
	}

}
