import java.util.ArrayList;
import java.util.Arrays;


public class Kromosom implements Comparable<Kromosom> {
	
	/** Zapis rješenja kojeg predstavlja jedinka */
	private ArrayList<Grupa> solution;
	/** Cijena rezanja jedinke (služi za računanje dobrote) */
	private double cost;
	/** Broj komada materijala koji nisu dovoljno dobro izrezani */
	private int numberOfWasted;
	/** Dobrota jedinke */
	private double fitness;
		
	/**
	 * Konstruktor objekta
	 */
	public Kromosom() {
		super();
		solution = new ArrayList<Grupa>();
		numberOfWasted = 0;
		cost = 0;
	}
	
	/**
	 * Računa cijenu rezanja jedinke (služi za računanje dobrote)
	 */
	public void calculateCost(){
		cost = (cost + ((double) numberOfWasted)/solution.size())/solution.size();
	}
	
	/**
	 * Dodaje grupu u rješenje koje jedinka predstavlja
	 * 
	 * @param grupa grupa predmeta
	 * @param wasted zastavica da li se materijal smatra nedovoljno dobro izrezan
	 */
	public void addGroup(Grupa grupa, boolean wasted){
		solution.add(grupa);
		cost += ((double) grupa.getRemainder());
		if (wasted)
			numberOfWasted++;
	}
	
	/**
	 * Postavlja dobrotu jedinke
	 * 
	 * @param fittnes dobrota jedinke
	 */
	public void setFitness(double fittnes) {
		this.fitness = fittnes;
	}

	/**
	 * Vraća dobrotu jedinke
	 * 
	 * @return dobrota jedinke
	 */
	public double getFitness() {
		return fitness;
	}
	
	/**
	 * Vraća broj grupa predmeta tj. broj iskorištenih komada materijala
	 * 
	 * @return broj grupa za rezanje
	 */
	public int getNumberOfGroups() {
		return solution.size();
	}

	/**
	 * Vraća rješenje koje jedinka predstavlja. Grupe su sortirane uzlazno prema ostatku koji ostaje nakon rezanja
	 * 
	 * @return rješenje prikazano jedinkom
	 */
	public Grupa[] getSolution() {
		Grupa[] vrati = solution.toArray(new Grupa[0]);
		Arrays.sort(vrati);
		return vrati;
	}


	/**
	 * Vraća cijenu rezanja jedinke (služi za računanje dobrote)
	 * 
	 * @return cijena rezanja jedinke
	 */
	public double getCost() {
		return cost;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Kromosom jedinka) {
		if (this.cost < jedinka.getCost())
			return -1;
		else if (this.cost > jedinka.getCost())
			return 1;
		else
			return 0;
		
	}
	
}
