import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;


public class HKromosom implements Comparable<HKromosom> {
	
	/** Zapis rješenja kojeg predstavlja jedinka */
	private ArrayList<HGrupa> solution;
	/** Cijena rezanja jedinke (služi za računanje dobrote) */
	private double cost;
	/** Broj novih komada materijala koji su iskorišteni u rješenju */
	private int numberOfNewStock;
	/** Broj komada materijala koji nisu dovoljno dobro izrezani */
	private int numberOfWasted;
	/** Dobrota jedinke */
	private double fitness;
	
	/**
	 * Konstruktor objekta
	 */
	public HKromosom() {
		super();
		solution = new ArrayList<HGrupa>();
		numberOfNewStock = 0;
		numberOfWasted = 0;
		cost = 0;
	}
	
	/**
	 * Računa cijenu rezanja jedinke (služi za računanje dobrote)
	 */
	public void calculateCost(){
		cost = (cost + ((double) numberOfWasted)/solution.size())/solution.size();
	}
	
	/**
	 * Dodaje grupu u rješenje koje jedinka predstavlja
	 * 
	 * @param grupa grupa predmeta
	 * @param newStock zastavica da li je korišten novi komad materijala
	 * @param wasted zastavica da li se materijal smatra nedovoljno dobro izrezan
	 */
	public void addGroup(HGrupa grupa, boolean newStock, boolean wasted){
		solution.add(grupa);
		cost += ((double) grupa.getRemainder())/grupa.getStockLength();
		if (newStock)
			numberOfNewStock++;
		if (wasted)
			numberOfWasted++;
	}
	
	/**
	 * Postavlja dobrotu jedinke
	 * 
	 * @param fittnes dobrota jedinke
	 */
	public void setFitness(double fittnes) {
		this.fitness = fittnes;
	}

	/**
	 * Vraća dobrotu jedinke
	 * 
	 * @return dobrota jedinke
	 */
	public double getFitness() {
		return fitness;
	}
	
	/**
	 * Vraća broj novih komada materijala koji su iskorišteni u rješenju
	 * 
	 * @return broj iskorištenih novih komada materijala
	 */
	public int getNumberOfNewStock() {
		return numberOfNewStock;
	}

	/**
	 * Vraća broj grupa predmeta tj. broj iskorištenih komada materijala
	 * 
	 * @return broj grupa za rezanje
	 */
	public int getNumberOfGroups() {
		return solution.size();
	}

	/**
	 * Vraća rješenje koje jedinka predstavlja
	 * 
	 * @return rješenje prikazano jedinkom
	 */
	public HGrupa[] getSolution() {
		return solution.toArray(new HGrupa[0]);
	}
	
	/**
	 * Vraća rješenje koje jedinka predstavlja. Grupe su sortirane uzlazno prema ostatku koji ostaje nakon rezanja
	 * 
	 * @return rješenje prikazano jedinkom
	 */
	public HGrupa[] getSortedSolution() {
		HGrupa[] vrati = solution.toArray(new HGrupa[0]);
		Arrays.sort(vrati);
		return vrati;
	}


	/**
	 * Vraća cijenu rezanja jedinke (služi za računanje dobrote)
	 * 
	 * @return cijena rezanja jedinke
	 */
	public double getCost() {
		return cost;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(HKromosom jedinka) {
		if (this.cost < jedinka.getCost())
			return -1;
		else if (this.cost > jedinka.getCost())
			return 1;
		else
			return 0;
		
	}

	@Override
	public String toString() {
		HashMap<Integer, Integer> temp = new HashMap<Integer, Integer>();
		for(HGrupa i : solution){
			System.err.print("(");
			for(Integer j : i.getGroup()){
				if (temp.containsKey(j))
					temp.put(j, temp.get(j)+1);
				else
					temp.put(j, 1);
				System.err.print(j + " ");
			}
			System.err.print(") ");
		}
		return temp.toString();
	}
	
	/**
	 * Smanjuje broj komada materijala koji nisu dovoljno dobro izrezani
	 */
	public void smanjiWasted(){
		numberOfWasted--;
	}
	
}
