import java.util.ArrayList;
import java.util.Arrays;

public class Populacija {
	
	/** Populacija rješenja */
	private ArrayList<SAKromosom> populacija;
	/** Duljine jedinki (služi za provođenje proporcionalne selekcije) */
	private double duljine[];
	

	/**
	 * Konstruktor objekta
	 * 
	 * @param populacija populacija potencijalnih rješenja
	 */
	public Populacija(ArrayList<SAKromosom> populacija) {
		super();
		this.populacija = populacija;
		this.pripremiSelekciju();
	}
	
	/** 
	 * Funkcija za obavljanje svih priprema za izvođenje skalirane proporcionalne selekcije
	 */
	private void pripremiSelekciju() {
		double maxCost = 0;
		double sumOfScaledCost = 0;
		
		for(SAKromosom i : populacija){
			if (i.getCost() > maxCost){
				maxCost = i.getCost();
			}
		}
		
		for(SAKromosom i : populacija){
			double t = maxCost - i.getCost();
			sumOfScaledCost += t;
			i.setFitness(t);
		}
		
		this.duljine = new double[populacija.size()];//populacija.size();
		for(int i = 0 ; i < populacija.size() ; i++){
			duljine[i] = populacija.get(i).getFitness()/sumOfScaledCost;
		}
	}
	
	/** 
	 * Postavlja novu populaciju rješenja
	 * 
	 * @param populacija nova populacija rješenja
	 */
	public void setPopulacija(ArrayList<SAKromosom> populacija){
		this.populacija = populacija;
		this.pripremiSelekciju();
	}
	
	/** 
	 * Funkcija koja selektira kromosom/jedinku koristeći skaliranu proporcionalnu selekciju
	 * 
	 * @return odabrani kromosom/jedinka
	 */
	public SAKromosom selektiraj(){
		double random = Math.random();
		double suma = 0;
		for(int i = 0 ; i < populacija.size() ; i++){
			if (suma + duljine[i] > random){
				return populacija.get(i);
			}
			suma += duljine[i];
		}
		return populacija.get(populacija.size()-1);
	}
	
	/**
	 * Vraća populaciju rješenja sortiranu prema cijeni rezanja
	 * 
	 * @return populacija rješenja
	 */
	public SAKromosom[] getSortedPopulation(){
		SAKromosom[] rezultat = populacija.toArray(new SAKromosom[0]);
		Arrays.sort(rezultat);
		return rezultat;
	}

}
