import java.util.ArrayList;
import java.util.Arrays;


public class SAKromosom implements Comparable<SAKromosom> {
	
	/** Zapis rješenja kojeg predstavlja jedinka */
	private ArrayList<Grupa> solution;
	/** Cijena rezanja jedinke (služi za računanje dobrote) */
	private double cost;
	/** Broj novih komada materijala koji su iskorišteni u rješenju */
	private int numberOfNewStock;
	/** Broj komada materijala koji nisu dovoljno dobro izrezani */
	private int numberOfWasted;
	/** Dobrota jedinke */
	private double fitness;
	/** GA parametri jedinke (7 bitni gayevi kodovi križanja i mutacije) */
	private String parametri;
	
	/**
	 * Konstruktor objekta
	 * 
	 * @param parametri GA parametri jedinke (7 bitni gayevi kodovi križanja i mutacije)
	 */
	public SAKromosom(String parametri) {
		super();
		solution = new ArrayList<Grupa>();
		numberOfNewStock = 0;
		numberOfWasted = 0;
		cost = 0;
		this.parametri = parametri;
	}
	
	/**
	 * Računa cijenu rezanja jedinke (služi za računanje dobrote)
	 */
	public void calculateCost(){
		cost = (cost + ((double) numberOfWasted)/solution.size())/*/solution.size()*/;
	}
	
	/**
	 * Dodaje grupu u rješenje koje jedinka predstavlja
	 * 
	 * @param grupa grupa predmeta
	 * @param newStock zastavica da li je korišten novi komad materijala
	 * @param wasted zastavica da li se materijal smatra nedovoljno dobro izrezan
	 */
	public void addGroup(Grupa grupa, boolean newStock, boolean wasted){
		solution.add(grupa);
		cost += grupa.getRemainder()/grupa.getStockLength();
		if (newStock)
			numberOfNewStock++;
		if (wasted)
			numberOfWasted++;
	}
	
	/**
	 * Postavlja dobrotu jedinke
	 * 
	 * @param fittnes dobrota jedinke
	 */
	public void setFitness(double fittnes) {
		this.fitness = fittnes;
	}

	/**
	 * Vraća dobrotu jedinke
	 * 
	 * @return dobrota jedinke
	 */
	public double getFitness() {
		return fitness;
	}
	
	/**
	 * Vraća broj novih komada materijala koji su iskorišteni u rješenju
	 * 
	 * @return broj iskorištenih novih komada materijala
	 */
	public int getNumberOfNewStock() {
		return numberOfNewStock;
	}

	/**
	 * Vraća broj grupa predmeta tj. broj iskorištenih komada materijala
	 * 
	 * @return broj grupa za rezanje
	 */
	public int getNumberOfGroups() {
		return solution.size();
	}

	/**
	 * Vraća rješenje koje jedinka predstavlja. Grupe su sortirane uzlazno prema ostatku koji ostaje nakon rezanja
	 * 
	 * @return rješenje prikazano jedinkom
	 */
	public Grupa[] getSolution() {
		Grupa[] vrati = solution.toArray(new Grupa[0]);
		Arrays.sort(vrati);
		return vrati;
	}


	/**
	 * Vraća GA parametre zapisane u jedinci
	 * 
	 * @return GA parametri
	 */
	public String getParametri() {
		return parametri;
	}

	/**
	 * Vraća cijenu rezanja jedinke (služi za računanje dobrote)
	 * 
	 * @return cijena rezanja jedinke
	 */
	public double getCost() {
		return cost;
	}
	
	/** Vraća vjerojatnost križanja zapisanu u jedinci
	 * 
	 * @return vjerojatnost križanja
	 */
	public double vjerojatnostKrizanja(){
		String gray = parametri.substring(0, 7);
		int dec = GrayCode.grayToDec(gray);
		return dec*0.01;
	}
	
	/** Vraća vjerojatnost mutacije zapisanu u jedinci
	 * 
	 * @return vjerojatnost mutacije
	 */
	public double vjerojatnostMutacije(){
		String gray = parametri.substring(7);
		int dec = GrayCode.grayToDec(gray);
		return dec*0.01;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(SAKromosom jedinka) {
		if (this.cost < jedinka.getCost())
			return -1;
		else if (this.cost > jedinka.getCost())
			return 1;
		else
			return 0;
		
	}
}
